import java.util.Iterator;
import java.util.NoSuchElementException;

public class LinkedList<E> implements Iterable<E> {

	private ListNode firstNode;
	private int size;

	public LinkedList() {
		firstNode = new ListNode(null);
		size = 0;
	}

	public boolean add(E e) {

		ListNode newNode = new ListNode(e);

		ListNode curNode = firstNode;
		while (curNode.nextNode != null) {
			curNode = curNode.nextNode;
		}
		// curNode = last node in our list
		curNode.nextNode = newNode;

		size++;

		return true;
	}

	public void add(int index, E e) {

		if (index < 0 || index > size) {
			throw new IndexOutOfBoundsException();
		}

		ListNode newNode = new ListNode(e);

		ListNode curNode = firstNode;
		for (int i = 0; i < index; i++) {
			curNode = curNode.nextNode;
		}
		// curNode is at index-1
		newNode.nextNode = curNode.nextNode;
		curNode.nextNode = newNode;

		size++;

	}

	public E remove() {
		return remove(0);
	}

	public E remove(int index) {

		if (index < 0 || index >= size) {
			throw new IndexOutOfBoundsException();
		}

		E toReturn;
		if (index == 0) {
			toReturn = firstNode.value;
			firstNode = firstNode.nextNode;
		} else {
			ListNode curNode = firstNode;
			for (int i = 0; i < index - 1; i++) {
				curNode = curNode.nextNode;
			}
			// curNode is at index-1
			toReturn = curNode.nextNode.value;
			curNode.nextNode = curNode.nextNode.nextNode;
		}

		size--;

		return toReturn;
	}

	public boolean remove(E e) {

		if (firstNode == null) {
			return false;
		} else if (firstNode.value.equals(e)) {
			firstNode = firstNode.nextNode;
			size--;
			return true;
		} else {
			ListNode curNode = firstNode;
			while (curNode.nextNode != null && !curNode.nextNode.value.equals(e)) {
				curNode = curNode.nextNode;
			}

			// curNode is just before the node we want to remove
			// curNode is the last node --> e does not exist in the list
			if (curNode.nextNode == null) {
				return false;
			} else {
				curNode.nextNode = curNode.nextNode.nextNode;
				size--;
				return true;
			}
		}
	}

	@Override
	public Iterator<E> iterator() {
		return new ListIterator();
	}

	private class ListIterator implements Iterator<E> {

		private ListNode curNode;

		public ListIterator() {
			curNode = firstNode;
		}

		@Override
		public boolean hasNext() {
			// if(curNode.nextNode != null) {
			// return true;
			// }
			// return false;

			return curNode.nextNode != null;
		}

		@Override
		public E next() {
			if (!hasNext()) {
				throw new NoSuchElementException();
			}

			curNode = curNode.nextNode;
			return curNode.value;
		}

	}

	public Iterator<E> evenIterator() {
		return new EvenListIterator();
	}

	private class EvenListIterator implements Iterator<E> {

		private ListNode curNode;

		public EvenListIterator() {
			ListNode sentinelNode = new ListNode(null);
			sentinelNode.nextNode = firstNode;
			curNode = sentinelNode;
		}

		@Override
		public boolean hasNext() {
			if (curNode.nextNode != null && curNode.nextNode.nextNode != null) {
				return true;
			}
			return false;
		}

		@Override
		public E next() {
			if (!hasNext()) {
				throw new NoSuchElementException();
			}

			curNode = curNode.nextNode.nextNode;
			return curNode.value;
		}

	}

	private class ListNode {

		private E value;
		private ListNode nextNode;

		public ListNode(E e) {
			value = e;
			nextNode = null;
		}

		public void setNext(ListNode next) {
			nextNode = next;
		}

		public ListNode getNext() {
			return nextNode;
		}

		public E getValue() {
			return value;
		}
	}

}
